@@ -1,4 +1,4 @@
-# $Id: File.pm,v 1.13 2005/12/31 21:33:58 comdog Exp $
+# $Id: File.pm,v 1.14 2006/03/08 18:34:08 comdog Exp $
 package Test::File;
 use strict;
 
@@ -6,7 +6,7 @@ use base qw(Exporter);
 use vars qw(@EXPORT $VERSION);
 
 use File::Spec;
-use Test::Builder 0.32;
+use Test::Builder;
 
 @EXPORT = qw(
 	file_exists_ok file_not_exists_ok
@@ -19,9 +19,10 @@ use Test::Builder 0.32;
 	symlink_target_dangles_ok
 	link_count_is_ok link_count_gt_ok link_count_lt_ok
 	owner_is owner_isnt
+	group_is group_isnt
 	);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/;
 
 my $Test = Test::Builder->new();
 
@@ -77,9 +78,21 @@ sub _no_symlinks_here { ! eval { symlink("",""); 1 } }
 #   Note:  I don't have a dos or mac os < 10 machine to test this on
 sub _obviously_non_multi_user 
 	{
-	($^O eq 'dos') ?	return 1	:
-	($^O eq 'MacOS') ?	return 1	:
-	return;	
+	($^O eq 'dos')   ?	
+		return 1	
+			:
+	($^O eq 'MacOS') ?	
+		return 1	
+			:
+		return;	
+
+	eval { my $holder = getpwuid(0) };
+	return 1 if $@; 
+
+	eval { my $holder = getgrgid(0) };
+	return 1 if $@;
+
+	return 0;	
 	}
 
 =over 4
@@ -793,6 +806,38 @@ sub link_count_lt_ok
 	$Test->ok( 1, $name );
 	}
 
+
+# owner_is, owner_isnt, group_is and group_isnt are almost 
+# identical in the beginning, so I'm writing a skeleton they can all use.
+# I can't think of a better name... 
+sub _dm_skeleton 
+	{ 
+	if( _obviously_non_multi_user() ) 
+		{
+		my $calling_sub = (caller(1))[3];
+		$Test->skip( $calling_sub . " only works on a multi-user OS" );	
+		return 'skip';
+		}
+
+	my $filename      = _normalize( shift );
+	my $testing_for   = shift;
+	my $name          = shift;
+	
+	unless( defined $filename ) 
+		{
+		$Test->diag( "File name not specified" );
+		return $Test->ok( 0, $name );
+		}
+
+	unless( -e $filename ) 
+		{
+		$Test->diag( "File [$filename] does not exist" );
+		return $Test->ok( 0, $name );
+		}
+
+	return;
+	}
+
 =item owner_is( FILE , OWNER [, NAME] )
 
 Ok if FILE's owner is the same as OWNER.  OWNER may be a text user name 
@@ -807,27 +852,13 @@ Contributed by Dylan Martin
 
 sub owner_is	 
 	{
-	if( _obviously_non_multi_user() ) 
-		{
-		$Test->skip( "owner_is needs a multi-user OS" );	
-		return;
-		}
-		
-	my $filename = _normalize( shift );
-	my $owner    = shift;
-	my $name     = shift || "$filename belongs to $owner";
+	my $filename      = shift;
+	my $owner         = shift;
+	my $name          = shift || "$filename belongs to $owner";
 
-	unless ( defined $filename ) 
-		{
-		$Test->diag( "File name not specified" );
-		return $Test->ok( 0, $name );
-		}
-
-	unless ( -e $filename ) 
-		{
-		$Test->diag( "File [$filename] does not exist" );
-		return $Test->ok( 0, $name );
-		}
+	my $err = _dm_skeleton( $filename, $owner, $name );
+	return if( defined( $err ) && $err eq 'skip' );
+	return $err if defined($err); 		
 
 	my $owner_uid = _get_uid( $owner );
 		
@@ -867,27 +898,13 @@ Contributed by Dylan Martin
 
 sub owner_isnt	 
 	{
-	if( _obviously_non_multi_user() ) 
-		{
-		$Test->skip( "owner_is needs a multi-user OS" );	
-		return;
-		}
-		
-	my $filename = _normalize( shift );
-	my $owner    = shift;
-	my $name     = shift || "$filename does not belong to $owner";
+	my $filename      = shift;
+	my $owner         = shift;
+	my $name          = shift || "$filename belongs to $owner";
 
-	unless( defined $filename ) 
-		{
-		$Test->diag( "Did not specify a file" );
-		return $Test->ok( 0, $name );
-		}
-
-	unless( -e $filename ) 
-		{
-		$Test->diag( "File [$filename] does not exist" );
-		return $Test->ok( 0, $name );
-		}
+	my $err = _dm_skeleton( $filename, $owner, $name );
+	return if( defined( $err ) && $err eq 'skip' ); 
+	return $err if defined($err); 		
 
 	my $owner_uid = _get_uid( $owner );
 	my $file_uid  = ( stat $filename )[4];
@@ -898,6 +915,91 @@ sub owner_isnt
 	return $Test->ok( 0, $name );
 	}
 
+=item group_is( FILE , GROUP [, NAME] )
+
+Ok if FILE's group is the same as GROUP.  GROUP may be a text group name or 
+a numeric group id.  Test skips on Dos, Mac OS <= 9 and any other operating
+systems that do not support getpwuid() and friends.  If the file does not
+exist, the test fails.
+ 
+The optional NAME parameter is the name of the test.
+  
+Contributed by Dylan Martin
+
+=cut
+
+sub group_is 
+ 	{
+	my $filename      = shift;
+	my $group         = shift;
+	my $name          = ( shift || "$filename belongs to group $group" );
+
+	my $err = _dm_skeleton( $filename, $group, $name );
+	return if( defined( $err ) && $err eq 'skip' );
+	return $err if defined($err); 		
+ 		
+	my $group_gid = _get_gid( $group );
+	my $file_gid  = ( stat $filename )[5];
+
+	unless( defined $file_gid ) 
+ 		{
+		$Test->skip("stat failed to return group gid for $filename");
+		return;
+		}
+
+	return $Test->ok( 1, $name ) if $file_gid == $group_gid; 
+
+	my $real_group = ( getgrgid $file_gid )[0];
+	unless( defined $real_group ) 
+		{
+		$Test->diag("File does not belong to $group");
+ 		return $Test->ok( 0, $name );
+ 		}
+ 
+	$Test->diag( "File [$filename] belongs to $real_group ($file_gid), ".
+			"not $group ($group_gid)" );
+
+	return $Test->ok( 0, $name );
+	}
+
+=item group_isnt( FILE , GROUP [, NAME] )
+
+Ok if FILE's group is not the same as GROUP.  GROUP may be a text group name or
+a numeric group id.  Test skips on Dos, Mac OS <= 9 and any other operating
+systems that do not support getpwuid() and friends.  If the file does not
+exist, the test fails.
+
+The optional NAME parameter is the name of the test.
+
+Contributed by Dylan Martin
+
+=cut
+
+sub group_isnt 
+	{
+	my $filename      = shift;
+	my $group         = shift;
+	my $name          = shift || "$filename does not belong to group $group";
+
+	my $err = _dm_skeleton( $filename, $group, $name );
+	return if( defined( $err ) && $err eq 'skip' );
+	return $err if defined($err); 		
+ 	
+	my $group_gid = _get_gid( $group );
+	my $file_gid  = ( stat $filename )[5];
+
+	unless( defined $file_gid ) 
+		{
+		$Test->skip("stat failed to return group gid for $filename");
+		return;
+		}
+
+	return $Test->ok( 1, $name ) if $file_gid != $group_gid; 
+
+	$Test->diag( "File [$filename] belongs to $group ($group_gid)" );
+ 		return $Test->ok( 0, $name );
+	}
+
 sub _get_uid
 	{
 	my $owner = shift;
@@ -915,6 +1017,24 @@ sub _get_uid
 		
 	$owner_uid;
 	}
+
+sub _get_gid
+	{
+	my $group = shift;
+	my $group_uid;
+	
+	if ($group =~ /^\d+/) 
+		{
+		$group_uid = $group;
+		$group = ( getgrgid $group )[0];
+		} 
+	else 
+		{
+		$group_uid = (getgrnam($group))[2];
+		}
+		
+	$group_uid;
+	}
 	
 =back
 
@@ -922,8 +1042,6 @@ sub _get_uid
 
 * check properties for other users (readable_by_root, for instance)
 
-* check group
-
 * check times
 
 * check number of links to file
